import os
import mimetypes
from webob import Response
from pyramid.asset import abspath_from_asset_spec

class NoFilenamesProvided(Exception):
    pass

def asset_view_factory(body, content_type, last_modified, http_cache=None):
    """ Returns an asset_view function that returns a SimpleResponse object"""
    response = Response()
    response.body = body
    response.content_type = content_type
    response.last_modified = last_modified
    response.cache_expires = http_cache
    def asset_view(context, request):
        return response
    return asset_view

def add_asset_views(config, spec, filename=None, filenames=None, http_cache=None):
    """ A pyramid directive that takes an asset spec and a file name or list of
    filenames and adds them to the pyramid config via the `add_view` function.

    These files added by this directive will be served relative to the Root
    of the application.  For Example:

    config.add_asset_views("mymodule:static", "favicon.ico")

    Alternatively you could use argument expansion with longer file lists:

    filenames = ['favicon.ico', 'robots.txt', 'humans.txt', 'crossdomain.xml']
    config.add_asset_views("mymodule:static", filenames=filenames)

    favicon.ico and robots.txt located in mymodule/static/ will now be served
    from /favicon.ico and /robots.txt respectively.

    Optionally the http_cache argument may be set with a number that represents
    a length of time in seconds.  For Example:

    config.add_asset_views("mymodule:static", "favicon.ico", http_cache=50000)

    Large files are not meant to be served this way as the current
    implementation reads the entire file into memory before generating the
    response object.  The main purpose of this module is to include assets
    that must be served from the root of the domain such as favicon.ico,
    robots.txt, and crossdomain.xml.
    """
    if filename is None and filenames is None:
        raise NoFilenamesProvided

    path = abspath_from_asset_spec(spec)

    if not filenames:
        filenames = []

    if filename:
        filenames.append(filename)

    for name in filenames:
        content_type = mimetypes.guess_type(name, strict=False)[0]
        if content_type is None:
            content_type = 'application/octet-stream'
        filename = os.path.join(path, name)
        body = open(filename, 'rb').read()
        last_modified = os.stat(filename).st_mtime
        config.add_view(asset_view_factory(body, content_type, last_modified,
                               http_cache=http_cache), name=name)

def includeme(config):
    """ Adds the add_asset_views directive into a pyramid applications config
    """
    config.add_directive('add_asset_views', add_asset_views)



import unittest
from pyramid import testing

class TestAssetViewConfig(unittest.TestCase):

    def setUp(self):
        self.request = testing.DummyRequest()
        self.config = testing.setUp(request=self.request)
        self.request.registry = self.config.registry
        import os
        here = os.path.abspath(os.path.dirname(__file__))
        self.test_assets = os.path.join(here, 'test_assets')

    def tearDown(self):
        testing.tearDown()
        del self.config

    def test_config(self):
        self.assertTrue(self.config is not None)

    def test_directive_added(self):
        self.config.include("pyramid_assetviews")
        self.assertTrue(self.config.add_asset_views is not None)


class TestAssetView(unittest.TestCase):

    def setUp(self):
        self.request = testing.DummyRequest()
        self.config = testing.setUp(request=self.request)
        self.request.registry = self.config.registry
        import os
        here = os.path.abspath(os.path.dirname(__file__))
        self.test_assets = os.path.join(here, 'test_assets')

    def tearDown(self):
        testing.tearDown()
        del self.config

    def test_no_filenames_exception(self):
        from pyramid_assetviews import add_asset_views
        from pyramid_assetviews import NoFilenamesProvided
        with self.assertRaises(NoFilenamesProvided):
            add_asset_views(self.config, 'pyramid_assetviews:test_assets')

    def test_asset_view_factory(self):
        from pyramid_assetviews import asset_view_factory
        asset_view = asset_view_factory("body", "image/x-icon", 124432, http_cache=500)
        response = asset_view("context", self.request)
        self.assertEqual(response.app_iter, ['body'])
        self.assertEqual(response.content_type, 'image/x-icon')
        self.assertTrue(response.last_modified is not None)
        self.assertTrue(response.expires is not None)
        self.assertTrue(response.cache_control.max_age, 500)

    def test_add_asset_views(self):
        from pyramid_assetviews import add_asset_views
        asset_spec = 'pyramid_assetviews:test_assets'
        filenames = ['favicon.ico', 'crossdomain.xml', 'robots.txt']
        add_asset_views(self.config, asset_spec, filenames=filenames)
        add_asset_views(self.config, asset_spec, 'robots.txt')
        add_asset_views(self.config, asset_spec, 'robots.txt', http_cache=500)

    def test_add_asset_views_with_no_mimetype(self):
        from pyramid_assetviews import add_asset_views
        asset_spec = 'pyramid_assetviews:test_assets'
        filenames = ['nomimetype']
        add_asset_views(self.config, asset_spec, filenames=filenames)



import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid',
    'webob',
    ]

if sys.version_info[:3] < (2,5,0):
    requires.append('pysqlite')

setup(name='pyramid_assetviews',
      version='1.0a2',
      description='pyramid_assetviews',
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        "Programming Language :: Python",
        ],
      author='Charlie Choiniere',
      author_email='pylons-discuss@googlegroups.com',
      url='https://github.com/nek4life/pyramid_assetviews',
      keywords='pyramid',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='pyramid_assetviews',
      install_requires = requires,
      )

